@@ -2,6 +2,7 @@ from typing import Any, Dict, List, Union
 
 import pipelime.choixe.ast.nodes as ast
 from pipelime.choixe.ast.parser import DIRECTIVE_PREFIX
+import astunparse
 
 
 class Unparser(ast.NodeVisitor):
@@ -117,6 +118,14 @@ class Unparser(ast.NodeVisitor):
             args.append(node.name)
         return self._unparse_auto("tmp", *args)
 
+    def visit_rand(self, node: ast.RandNode) -> Any:
+        kwargs = {}
+        if node.n is not None:
+            kwargs["n"] = node.n
+        if node.pdf is not None:
+            kwargs["pdf"] = node.pdf
+        return self._unparse_auto("rand", *node.args, **kwargs)
+
     def _unparse_as_arg(self, node: ast.Node) -> str:
         unparsed = node.accept(self)
         if isinstance(unparsed, str):
@@ -124,6 +133,8 @@ class Unparser(ast.NodeVisitor):
                 return unparsed
             else:
                 return f'"{unparsed}"'
+        elif callable(unparsed) and callable.__name__ == "<lambda>":  # pragma: no cover
+            raise ValueError("Cannot unparse lambda expressions to string.")
         else:
             return str(unparsed)
 
