@@ -9,6 +9,18 @@ from pipelime.sequences.sample import Sample
 
 if t.TYPE_CHECKING:
     from pipelime.utils.pydantic_types import SampleValidationInterface
+    from pipelime.sequences.direct_access import DirectAccessSequence
+    from pipelime.stages import SampleStage, StageInput
+
+
+def _sseq_stub_dummy(*args, **kwargs):
+    raise NotImplementedError(
+        "This is a stub function. It is not supposed to be called."
+    )
+
+
+def samples_sequence_stub(func):
+    return _sseq_stub_dummy
 
 
 class SamplesSequenceBase(t.Sequence[Sample]):
@@ -97,7 +109,7 @@ class SamplesSequence(
             return cls.__config__.title
         return cls.__name__
 
-    def direct_access(self) -> t.Sequence[t.Mapping[str, t.Any]]:
+    def direct_access(self) -> "DirectAccessSequence":
         """Returns a sequence of key-to-value mappings,
         with no intermediate Sample and Item classes.
         """
@@ -126,7 +138,7 @@ class SamplesSequence(
         *,
         num_workers: int = 0,
         prefetch: int = 2,
-        track_fn: t.Optional[t.Callable[[t.Iterable], t.Iterable]] = None,
+        track_fn: t.Union[bool, str, t.Callable[[t.Iterable], t.Iterable], None] = True,
     ):
         """Goes through all the samples of the sequence, optionally using multiple
         processes and returns a new sequence holding the processed samples.
@@ -140,13 +152,16 @@ class SamplesSequence(
         :type prefetch: int, optional
         :param sample_fn: a callable to run on each sample, defaults to None
         :type sample_fn: t.Optional[t.Callable[[Sample], None]], optional
-        :param track_fn: a callable to track the progress, defaults to None
-        :type track_fn: t.Optional[t.Callable[[t.Iterable], t.Iterable]], optional
+        :param track_fn: if True, a rich trackbar is shown; if a string is passed, it is
+            set as the message for the default rich trackbar; otherwise you should
+            provide your own callable to track the progress, defaults to True
+        :type track_fn: t.Union[bool, str, t.Callable[[t.Iterable], t.Iterable], None],
+            optional
         """
         samples: t.List[Sample] = []
 
         def _store_sample(x: Sample, idx: int):
-            if len(samples) <= idx:
+            if len(samples) <= idx:  # pragma: no branch
                 samples.extend([Sample() for _ in range(idx - len(samples) + 1)])
             samples[idx] = x
 
@@ -169,7 +184,7 @@ class SamplesSequence(
         sample_fn: t.Union[
             t.Callable[[Sample], None], t.Callable[[Sample, int], None], None
         ] = None,
-        track_fn: t.Optional[t.Callable[[t.Iterable], t.Iterable]] = None,
+        track_fn: t.Union[bool, str, t.Callable[[t.Iterable], t.Iterable], None] = True,
     ):
         """Goes through all the samples of the sequence, optionally using multiple
         processes and applying `sample_fn` to each sample. Also, a `track_fn` can be
@@ -186,15 +201,31 @@ class SamplesSequence(
         :type keep_order: bool, optional
         :param sample_fn: a callable to run on each sample, defaults to None
         :type sample_fn: t.Optional[t.Callable[[Sample], None]], optional
-        :param track_fn: a callable to track the progress, defaults to None
-        :type track_fn: t.Optional[t.Callable[[t.Iterable], t.Iterable]], optional
+        :param track_fn: if True, a rich trackbar is shown; if a string is passed, it is
+            set as the message for the default rich trackbar; otherwise you should
+            provide your own callable to track the progress, defaults to True
+        :type track_fn: t.Union[bool, str, t.Callable[[t.Iterable], t.Iterable], None],
+            optional
         """
         from pipelime.sequences import Grabber, grab_all
 
+        if isinstance(track_fn, (bool, str)):
+            import rich.progress
+
+            if track_fn:
+                msg = track_fn if isinstance(track_fn, str) else ""
+                track_fn = lambda x: rich.progress.track(
+                    x,
+                    total=len(self),
+                    description=msg,
+                )
+            else:
+                track_fn = None
+
         grabber = Grabber(
             num_workers=num_workers, prefetch=prefetch, keep_order=keep_order
         )
-        grab_all(grabber, self, sample_fn=sample_fn, track_fn=track_fn)
+        grab_all(grabber, self, sample_fn=sample_fn, track_fn=track_fn)  # type: ignore
 
     def to_pipe(
         self, recursive: bool = True, objs_to_str: bool = True
@@ -262,6 +293,7 @@ class SamplesSequence(
     # FUNCTION STUBS FOR TYPE CHECKING AND AUTO-COMPLETION
     ###########################################################################
 
+    @samples_sequence_stub
     @staticmethod
     def from_callable(
         *,
@@ -273,6 +305,7 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     @staticmethod
     def from_list(samples: t.Sequence[Sample]) -> SamplesSequence:
         """A SamplesSequence from a list of Samples.
@@ -280,6 +313,7 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     @staticmethod
     def from_underfolder(
         folder: "pathlib.Path",  # type: ignore # noqa: E602,F821
@@ -293,6 +327,7 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     @staticmethod
     def toy_dataset(
         length: int,
@@ -314,10 +349,14 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     def map(
         self,
         stage: t.Union[
-            "pipelime.stages.SampleStage",  # type: ignore # noqa: E602,F821
+            "StageInput",
+            "SampleStage",
+            str,
+            bytes,
             t.Mapping[str, t.Optional[t.Mapping[str, t.Any]]],
         ],
     ) -> SamplesSequence:
@@ -326,30 +365,64 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
+    def map_if(
+        self,
+        stage: t.Union[
+            "StageInput",
+            "SampleStage",
+            str,
+            bytes,
+            t.Mapping[str, t.Optional[t.Mapping[str, t.Any]]],
+        ],
+        condition: t.Union[
+            t.Callable[[], bool],
+            t.Callable[[int], bool],
+            t.Callable[[int, Sample], bool],
+            t.Callable[[int, Sample, SamplesSequence], bool],
+        ],
+    ) -> SamplesSequence:
+        """Applies a stage on all samples if a condition returns True.
+        Run `pipelime help map_if` to read the complete documentation.
+        """
+        ...
+
+    @samples_sequence_stub
     def zip(self, to_zip: SamplesSequence, *, key_format: str = "*") -> SamplesSequence:
         """Zips two Sequences by merging each Sample.
         Run `pipelime help zip` to read the complete documentation.
         """
         ...
 
+    @samples_sequence_stub
     def cat(self, to_cat: SamplesSequence) -> SamplesSequence:
         """Concatenates two SamplesSequences.
         Run `pipelime help cat` to read the complete documentation.
         """
         ...
 
-    def filter(self, filter_fn: t.Callable[[Sample], bool]) -> SamplesSequence:
+    @samples_sequence_stub
+    def filter(
+        self,
+        filter_fn: t.Callable[[Sample], bool],
+        lazy: bool = True,
+        insert_empty_samples: bool = False,
+    ) -> SamplesSequence:
         """A filtered view of a SamplesSequence.
         Run `pipelime help filter` to read the complete documentation.
         """
         ...
 
-    def sort(self, key_fn: t.Callable[[Sample], t.Any]) -> SamplesSequence:
+    @samples_sequence_stub
+    def sort(
+        self, key_fn: t.Callable[[Sample], t.Any], lazy: bool = True
+    ) -> SamplesSequence:
         """A sorted view of an input SamplesSequence.
         Run `pipelime help sort` to read the complete documentation.
         """
         ...
 
+    @samples_sequence_stub
     def slice(
         self,
         *,
@@ -362,6 +435,7 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     def select(
         self, indexes: t.Sequence[int], *, negate: bool = False
     ) -> SamplesSequence:
@@ -371,12 +445,14 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     def shuffle(self, *, seed: t.Optional[int] = None) -> SamplesSequence:
         """Shuffles samples in the input SamplesSequence.
         Run `pipelime help shuffle` to read the complete documentation.
         """
         ...
 
+    @samples_sequence_stub
     def enumerate(
         self,
         *,
@@ -388,12 +464,14 @@ class SamplesSequence(
         """
         ...
 
-    def repeat(self, count: int) -> SamplesSequence:
+    @samples_sequence_stub
+    def repeat(self, count: int, interleave: bool = False) -> SamplesSequence:
         """Repeat this sequence so each sample is seen multiple times.
         Run `pipelime help repeat` to read the complete documentation.
         """
         ...
 
+    @samples_sequence_stub
     def cache(
         self,
         cache_folder: t.Optional["pathlib.Path"] = None,  # type: ignore # noqa: E602,F821
@@ -405,18 +483,21 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     def data_cache(
         self, *items: t.Union[t.Type["pipelime.items.Item"], str]  # type: ignore # noqa: E602,F821
     ) -> SamplesSequence:
         """Enables item data caching on previous pipeline steps."""
         ...
 
+    @samples_sequence_stub
     def no_data_cache(
         self, *items: t.Union[t.Type["pipelime.items.Item"], str]  # type: ignore # noqa: E602,F821
     ) -> SamplesSequence:
         """Disables item data caching on previous pipeline steps."""
         ...
 
+    @samples_sequence_stub
     def to_underfolder(
         self,
         folder: "pathlib.Path",  # type: ignore # noqa: E602,F821
@@ -434,6 +515,7 @@ class SamplesSequence(
         """
         ...
 
+    @samples_sequence_stub
     def validate_samples(
         self, *, sample_schema: "SampleValidationInterface"
     ) -> SamplesSequence:
