@@ -1,6 +1,7 @@
 import os
 import typing as t
 from pathlib import Path
+from loguru import logger
 
 import pydantic as pyd
 
@@ -31,10 +32,10 @@ class UnderfolderReader(pls.SamplesSequence, title="from_underfolder"):
         ),
     )
 
-    _samples: t.List[t.Union[pls.Sample, t.Dict[str, Path]]] = pyd.PrivateAttr(
+    _samples: t.List[t.Union[pls.Sample, t.Dict[str, str]]] = pyd.PrivateAttr(
         default_factory=list
     )
-    _root_sample: t.Optional[t.Union[pls.Sample, t.Dict[str, Path]]] = pyd.PrivateAttr(
+    _root_sample: t.Optional[t.Union[pls.Sample, t.Dict[str, str]]] = pyd.PrivateAttr(
         None
     )
 
@@ -74,16 +75,22 @@ class UnderfolderReader(pls.SamplesSequence, title="from_underfolder"):
 
     def _extract_id_key(self, name: str) -> t.Optional[t.Tuple[int, str]]:
         id_key_split = name.partition("_")
-        if not id_key_split[2]:
-            return None  # pragma: no cover
+        if not id_key_split[2]:  # pragma: no cover
+            logger.warning(
+                f"{self.__class__}: cannot parse file name {name} as <id>_<key>.<ext>"
+            )
+            return None
         try:
             return (int(id_key_split[0]), self._extract_key(id_key_split[2]))
         except ValueError:  # pragma: no cover
+            logger.warning(
+                f"{self.__class__}: file name `{name}` does not start with an integer"
+            )
             return None
 
     def _scan_root_files(self):
         if self.folder.exists():
-            root_items: t.Dict[str, Path] = {}
+            root_items: t.Dict[str, str] = {}
             with os.scandir(str(self.folder)) as it:
                 for entry in it:
                     if entry.is_file():
@@ -91,22 +98,31 @@ class UnderfolderReader(pls.SamplesSequence, title="from_underfolder"):
                         if key:
                             root_items[key] = entry.path
             self._root_sample = root_items if root_items else pls.Sample()
-        else:
+        else:  # pragma: no cover
+            logger.warning(
+                f"{self.__class__}: root folder `{self.folder}` does not exist"
+            )
             self._root_sample = pls.Sample()
 
     def _scan_sample_files(self):
         data_folder = self.folder / "data"
         if data_folder.exists():
-            self._samples = []
+            samples = []
             with os.scandir(str(data_folder)) as it:
                 for entry in it:
                     if entry.is_file():
                         id_key = self._extract_id_key(entry.name)
                         if id_key:
-                            self._samples.extend(
-                                [{} for _ in range(id_key[0] - len(self._samples) + 1)]
+                            samples.extend(
+                                ({} for _ in range(id_key[0] - len(samples) + 1))
                             )
-                            self._samples[id_key[0]][id_key[1]] = entry.path
+                            samples[id_key[0]][id_key[1]] = entry.path
+            self._samples = samples
+        else:  # pragma: no cover
+            logger.warning(
+                f"{self.__class__}: data folder `{data_folder}` does not exist"
+            )
+            self._samples = []
 
     def size(self) -> int:
         if self.watch:
@@ -128,5 +144,7 @@ class UnderfolderReader(pls.SamplesSequence, title="from_underfolder"):
                     for k, v in sample.items()
                 }
             )
+            if self.merge_root_items:
+                sample = self.root_sample.merge(sample)
             self._samples[idx] = sample
-        return self.root_sample.merge(sample) if self.merge_root_items else sample
+        return sample
